<?php

/**
 * @file
 * Migrate pages given in xml format.
 */

/**
 * Implementation of WordPressMigration, for pages.
 */

class WordPressPostMigration extends XMLMigration {

  public $contenttype = '';
  public $dependencies = array('WordPressCategory', 'WordPressTag',
    'WordPressUser');

  public $description = 'Wordpress post exported as XML';

  public $xmlfolder = 'wordpress/posts_xml';
  public $staticfolder = 'wordpress/static';

  public $defaultfilterformat = '';
  public $defaultuid = 1;
  public $defaultcommentsetting = 2;

  public $fields = array(
    'id' => 'Legacy ID',
    'legacy_url' => 'Legacy URL',
    'title' => 'Title',
    'body' => 'Body',
    'pubdate' => 'Publish Date',
    'authors' => 'Authors',
    'teaser' => 'Teaser',
    'categories' => 'Categories',
    'tags' => 'Tags',
    'status' => 'Publish Status',
    'urls' => 'Old Site Urls',
  );

  /**
   * Set it up : Initialisation.
   */
  public function __construct() {
    parent::__construct();
    $this->contenttype = variable_get('wp_migrate_post_type', 'story');
    $xml_path = file_directory_path() . '/' . $this->xmlfolder;
    $this->defaultfilterformat = variable_get('wp_migrate_text_format', 'filtered_html');
    // The source ID here is the one retrieved from the XML listing file, and
    // used to identify the specific item's file.
    $this->map = new MigrateSQLMap(
      $this->machineName,
      array(
        'sourceid' => array(
          'type' => 'int',
          'unsigned' => TRUE,
          'not null' => TRUE,
        ),
      ),
      MigrateDestinationNode::getKeySchema()
    );

    // Each ID retrieved from the list URL will be plugged into :id in the
    // item URL to fetch the specific objects.
    $item_url = $xml_path . '/:id.xml';
    // We use the MigrateSourceList class for any source where we have the list
    // of IDs to process separately from the data for each item. The listing
    // and item are represented by separate classes, so for example we could
    // replace the XML listing with a file directory listing, or the XML item
    // with a JSON item.
    $this->source = new MigrateSourceList(new WordpressMigrateListFiles($xml_path),
      new MigrateItemXML($item_url), $this->fields);
    $this->destination = new MigrateDestinationNode($this->contenttype);
    $this->setupmap();
  }

  /**
   * Custom function defining source and destination mapping.
   */
  public function setupmap() {

    $date_args = array(
      'timezone' => 'Asia/Calcutta',
      'language' => LANGUAGE_NONE);

    $static_path = file_directory_path() . '/' . $this->staticfolder;
    $static_args = MigrateFileFieldHandler::arguments($static_path);

    $this->addFieldMapping('format')
         ->defaultValue($this->defaultfilterformat);
    $this->addFieldMapping('uid')
         ->defaultValue($this->defaultuid);
    $this->addFieldMapping('comment')
         ->defaultValue($this->defaultcommentsetting);
    $this->addFieldMapping('status', 'status');
    $this->addFieldMapping('created', 'pubdate');
    $this->addFieldMapping('title', 'title');
    $this->addFieldMapping('body', 'body');
    $this->addFieldMapping('field_pubdate', 'pubdate')
      ->arguments($date_args);
    $this->addFieldMapping('teaser', 'excerpt');
    $this->addFieldMapping('Category', 'categories')
      ->sourceMigration('WordPressCategory')
      ->arguments(array('source_type' => 'tid'));
    $this->addFieldMapping('Keywords', 'tags')
      ->sourceMigration('WordPressTag');

    $this->addFieldMapping('path', 'legacy_url');
  }

  /**
   * Modify source row and get data from xml.
   */
  public function prepareRow($row) {
    $row->id = (string) $row->xml->doc_id['id'];
    $row->legacy_url = ltrim((string) $row->xml->primary_url['value'], '/');
    $row->title = (string) $row->xml->title;
    $row->body = (string) $row->xml->body;
    $row->pubdate = (string) $row->xml->published_date['iso'];

    if (!$row->pubdate || strtotime($row->pubdate) > time()) {
      // Not time to publish yet.
      $row->status = 0;
    }
    else {
      // Go ahead and publish.
      $row->status = 1;
    }
    $row->teaser = (string) $row->xml->excerpt;

    $row->authors = array();
    foreach ($row->xml->authors->author as $auth) {
      $row->authors[] = (string) $auth['username'];
    }

    $row->categories = array();
    foreach ($row->xml->categories->category as $cat) {
      $row->categories[] = (int) $cat['id'];
    }

    $row->tags = array();
    foreach ($row->xml->tags->tag as $key) {
      $row->tags[] = (int) $key['id'];
    }

    $row->urls = array();
    foreach ($row->xml->all_urls->url as $url) {
      $row->urls[] = (string) $url['value'];
    }
  }

  /**
   * Modify node object and add author nid and uid.
   */
  public function prepare($node, $row) {

    if (module_exists('scheduler')) {
      $now = time();
      if ($row->pubdate) {
        $pubdate_timestamp = strtotime($row->pubdate);
        if ($pubdate_timestamp > $now) {
          $node->publish_on = $pubdate_timestamp;
        }
      }
    }

    // On node_save()
    // so pre-emptively force pubdate to match created here.
    if (is_array($node->field_pubdate) && is_array($node->field_pubdate[0])) {
      $node->field_pubdate[0]['value'] = strftime('%Y-%m-%dT%H:%M:%S', $node->created);
    }
  }
}
